@@ -5,20 +5,19 @@
 */
 
 #include "transport.h"
-#include "transport_p.h"
 #include "mailtransport_defs.h"
+#include "transport_p.h"
 #include "transportmanager.h"
 #include "transporttype_p.h"
 
-#include <QTimer>
-
-#include <KConfigGroup>
 #include "mailtransport_debug.h"
+#include <KConfigGroup>
 #include <KLocalizedString>
 #include <KMessageBox>
 #include <KStringHandler>
 #include <KWallet/KWallet>
-
+#include <qt5keychain/keychain.h>
+using namespace QKeychain;
 using namespace MailTransport;
 using namespace KWallet;
 
@@ -31,8 +30,8 @@ Transport::Transport(const QString &cfgGroup)
     d->passwordDirty = false;
     d->storePasswordInFile = false;
     d->needsWalletMigration = false;
-    d->passwordNeedsUpdateFromWallet = false;
     load();
+    loadPassword();
 }
 
 Transport::~Transport()
@@ -45,12 +44,15 @@ bool Transport::isValid() const
     return (id() > 0) && !host().isEmpty() && port() <= 65536;
 }
 
-QString Transport::password()
+void Transport::loadPassword()
 {
-    if (!d->passwordLoaded && requiresAuthentication() && storePassword()
-        && d->password.isEmpty()) {
+    if (!d->passwordLoaded && requiresAuthentication() && storePassword() && d->password.isEmpty()) {
         readPassword();
     }
+}
+
+QString Transport::password()
+{
     return d->password;
 }
 
@@ -76,8 +78,12 @@ void Transport::forceUniqueName()
     int suffix = 1;
     QString origName = name();
     while (existingNames.contains(name())) {
-        setName(i18nc("%1: name; %2: number appended to it to make "
-                      "it unique among a list of names", "%1 #%2", origName, suffix));
+        setName(
+            i18nc("%1: name; %2: number appended to it to make "
+                  "it unique among a list of names",
+                  "%1 #%2",
+                  origName,
+                  suffix));
         ++suffix;
     }
 }
@@ -150,13 +156,13 @@ void Transport::usrRead()
     {
         d->transportType = TransportType();
         d->transportType.d->mIdentifier = identifier();
-        //qCDebug(MAILTRANSPORT_LOG) << "type" << identifier();
+        // qCDebug(MAILTRANSPORT_LOG) << "type" << identifier();
         // Now we have the type and possibly agentType.  Get the name, description
         // etc. from TransportManager.
         const TransportType::List &types = TransportManager::self()->types();
         int index = types.indexOf(d->transportType);
         if (index != -1) {
-            d->transportType = types[ index ];
+            d->transportType = types[index];
         } else {
             qCWarning(MAILTRANSPORT_LOG) << "Type unknown to manager.";
             d->transportType.d->mName = i18nc("An unknown transport type", "Unknown");
@@ -169,20 +175,6 @@ void Transport::usrRead()
     }
 
     if (d->passwordLoaded) {
-        if (d->passwordNeedsUpdateFromWallet) {
-            d->passwordNeedsUpdateFromWallet = false;
-            // read password if wallet is open, defer otherwise
-            if (Wallet::isOpen(Wallet::NetworkWallet())) {
-                // Don't read the password right away because this can lead
-                // to reentrancy problems in KDBusServiceStarter when an application
-                // run in Kontact creates the transports (due to a QEventLoop in the
-                // synchronous KWallet openWallet call).
-                QTimer::singleShot(0, this, &Transport::readPassword);
-            } else {
-                d->passwordLoaded = false;
-            }
-        }
-
         return;
     }
 
@@ -190,8 +182,6 @@ void Transport::usrRead()
     KConfigGroup group(config(), currentGroup());
     if (group.hasKey("password")) {
         d->password = KStringHandler::obscure(group.readEntry("password"));
-    } else if (group.hasKey("password-kmail")) { //Legacy
-        d->password = KStringHandler::obscure(group.readEntry("password-kmail"));
     }
 
     if (!d->password.isEmpty()) {
@@ -211,19 +201,21 @@ bool Transport::usrSave()
         Wallet *wallet = TransportManager::self()->wallet();
         if (!wallet || wallet->writePassword(QString::number(id()), d->password) != 0) {
             // wallet saving failed, ask if we should store in the config file instead
-            if (d->storePasswordInFile || KMessageBox::warningYesNo(
-                    nullptr,
-                    i18n("KWallet is not available. It is strongly recommended to use "
-                         "KWallet for managing your passwords.\n"
-                         "However, the password can be stored in the configuration "
-                         "file instead. The password is stored in an obfuscated format, "
-                         "but should not be considered secure from decryption efforts "
-                         "if access to the configuration file is obtained.\n"
-                         "Do you want to store the password for server '%1' in the "
-                         "configuration file?", name()),
-                    i18n("KWallet Not Available"),
-                    KGuiItem(i18n("Store Password")),
-                    KGuiItem(i18n("Do Not Store Password"))) == KMessageBox::Yes) {
+            if (d->storePasswordInFile
+                || KMessageBox::warningYesNo(nullptr,
+                                             i18n("KWallet is not available. It is strongly recommended to use "
+                                                  "KWallet for managing your passwords.\n"
+                                                  "However, the password can be stored in the configuration "
+                                                  "file instead. The password is stored in an obfuscated format, "
+                                                  "but should not be considered secure from decryption efforts "
+                                                  "if access to the configuration file is obtained.\n"
+                                                  "Do you want to store the password for server '%1' in the "
+                                                  "configuration file?",
+                                                  name()),
+                                             i18n("KWallet Not Available"),
+                                             KGuiItem(i18n("Store Password")),
+                                             KGuiItem(i18n("Do Not Store Password")))
+                    == KMessageBox::Yes) {
                 // write to config file
                 KConfigGroup group(config(), currentGroup());
                 group.writeEntry("password", KStringHandler::obscure(storePassword));
@@ -253,44 +245,22 @@ void Transport::readPassword()
     }
     d->passwordLoaded = true;
 
-    // check whether there is a chance to find our password at all
-    if (Wallet::folderDoesNotExist(Wallet::NetworkWallet(), WALLET_FOLDER)
-        || Wallet::keyDoesNotExist(Wallet::NetworkWallet(), WALLET_FOLDER,
-                                   QString::number(id()))) {
-        // try migrating password from kmail
-        if (Wallet::folderDoesNotExist(Wallet::NetworkWallet(), KMAIL_WALLET_FOLDER)
-            || Wallet::keyDoesNotExist(Wallet::NetworkWallet(), KMAIL_WALLET_FOLDER,
-                                       QStringLiteral("transport-%1").arg(id()))) {
-            return;
-        }
-        qCDebug(MAILTRANSPORT_LOG) << "migrating password from kmail wallet";
-        KWallet::Wallet *wallet = TransportManager::self()->wallet();
-        if (wallet) {
-            QString pwd;
-            wallet->setFolder(KMAIL_WALLET_FOLDER);
-            if (wallet->readPassword(QStringLiteral("transport-%1").arg(id()), pwd) == 0) {
-                setPassword(pwd);
-                save();
-            } else {
-                d->password.clear();
-                d->passwordLoaded = false;
-            }
-            wallet->removeEntry(QStringLiteral("transport-%1").arg(id()));
-            wallet->setFolder(WALLET_FOLDER);
-        }
-        return;
-    }
+    auto readJob = new ReadPasswordJob(WALLET_FOLDER, this);
+    connect(readJob, &Job::finished, this, &Transport::readTransportPasswordFinished);
+    readJob->setKey(QString::number(id()));
+    readJob->start();
+}
 
-    // finally try to open the wallet and read the password
-    KWallet::Wallet *wallet = TransportManager::self()->wallet();
-    if (wallet) {
-        QString pwd;
-        if (wallet->readPassword(QString::number(id()), pwd) == 0) {
-            setPassword(pwd);
-        } else {
-            d->password.clear();
-            d->passwordLoaded = false;
-        }
+void Transport::readTransportPasswordFinished(QKeychain::Job *baseJob)
+{
+    auto job = qobject_cast<ReadPasswordJob *>(baseJob);
+    Q_ASSERT(job);
+    if (job->error()) {
+        d->password.clear();
+        d->passwordLoaded = false;
+        qWarning() << "We have an error during reading password " << job->errorString();
+    } else {
+        setPassword(job->textData());
     }
 }
 
@@ -305,7 +275,6 @@ void Transport::migrateToWallet()
     d->needsWalletMigration = false;
     KConfigGroup group(config(), currentGroup());
     group.deleteEntry("password");
-    group.deleteEntry("password-kmail");
     d->passwordDirty = true;
     d->storePasswordInFile = false;
     save();
@@ -313,7 +282,7 @@ void Transport::migrateToWallet()
 
 Transport *Transport::clone() const
 {
-    QString id = currentGroup().mid(10);
+    const QString id = currentGroup().mid(10);
     return new Transport(id);
 }
 
